#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
    /// Measures the amount of minutes it takes for a tick/volume/range/etc. bar to form
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
	public class BarSpeed : Indicator
	{
		private SessionIterator sessionIterator;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "BarSpeed";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				
				AddPlot(new Stroke(Brushes.Blue, 4), PlotStyle.Bar, "TimePast");
				AddLine(Brushes.Black, 0, "Zero");
			}
			else if (State == State.DataLoaded)
			{
				sessionIterator = new SessionIterator(Bars);
			}
		}

		protected override void OnBarUpdate()
		{
			DateTime pTime = DateTime.MinValue;	
			if (Bars.IsFirstBarOfSession)
			{
				sessionIterator.GetNextSession(Time[0], true);
				pTime = sessionIterator.ActualSessionBegin;
			}
			else if (CurrentBar>0)
				pTime = Time[1];
			
			if (pTime.CompareTo(DateTime.MinValue)>0)
			{
				TimeSpan span = Time[0].Subtract(pTime);			
	            TimePast[0] = span.TotalMinutes;
			}
		}

		#region Properties

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> TimePast
		{
			get { return Values[0]; }
		}

		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private BarSpeed[] cacheBarSpeed;
		public BarSpeed BarSpeed()
		{
			return BarSpeed(Input);
		}

		public BarSpeed BarSpeed(ISeries<double> input)
		{
			if (cacheBarSpeed != null)
				for (int idx = 0; idx < cacheBarSpeed.Length; idx++)
					if (cacheBarSpeed[idx] != null &&  cacheBarSpeed[idx].EqualsInput(input))
						return cacheBarSpeed[idx];
			return CacheIndicator<BarSpeed>(new BarSpeed(), input, ref cacheBarSpeed);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.BarSpeed BarSpeed()
		{
			return indicator.BarSpeed(Input);
		}

		public Indicators.BarSpeed BarSpeed(ISeries<double> input )
		{
			return indicator.BarSpeed(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.BarSpeed BarSpeed()
		{
			return indicator.BarSpeed(Input);
		}

		public Indicators.BarSpeed BarSpeed(ISeries<double> input )
		{
			return indicator.BarSpeed(input);
		}
	}
}

#endregion
